package com.zhiLian.business.controller;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import com.zhiLian.business.domain.Business;
import com.zhiLian.business.service.IBusinessService;
import com.zhiLian.common.core.domain.Result;
import com.zhiLian.common.core.utils.poi.ExcelUtil;
import com.zhiLian.common.core.web.controller.BaseController;
import com.zhiLian.common.core.web.page.TableDataInfo;
import com.zhiLian.common.log.annotation.Log;
import com.zhiLian.common.log.enums.BusinessType;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;


/**
 * 企业Controller
 *
 * @author muyu
 * @date 2024-05-26
 */
@RestController
@RequestMapping("/business")
public class BusinessController extends BaseController
{
    @Autowired
    private IBusinessService businessService;

    /**
     * 查询企业列表
     */
//    @RequiresPermissions("system:business:list")
    @GetMapping("/list")
    public Result<TableDataInfo<Business>> list(Business business)
    {
        startPage();
        List<Business> list = businessService.selectBusinessList(business);
        return getDataTable(list);
    }

    /**
     * 导出企业列表
     */
//    @RequiresPermissions("system:business:export")
    @Log(title = "企业", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, Business business)
    {
        List<Business> list = businessService.selectBusinessList(business);
        ExcelUtil<Business> util = new ExcelUtil<Business>(Business.class);
        util.exportExcel(response, list, "企业数据");
    }

    /**
     * 获取企业详细信息
     */
//    @RequiresPermissions("system:business:query")
    @GetMapping(value = "/{id}")
    public Result getInfo(@PathVariable("id") Long id)
    {
        return success(businessService.selectBusinessById(id));
    }

    /**
     * 新增企业
     */
//    @RequiresPermissions("system:business:add")
    @Log(title = "企业", businessType = BusinessType.INSERT)
    @PostMapping
    public Result add(@RequestBody Business business)
    {
        return toAjax(businessService.insertBusiness(business));
    }

    /**
     * 修改企业
     */
//    @RequiresPermissions("system:business:edit")
    @Log(title = "企业", businessType = BusinessType.UPDATE)
    @PutMapping
    public Result edit(@RequestBody Business business)
    {
        return toAjax(businessService.updateBusiness(business));
    }

    /**
     * 删除企业
     */
//    @RequiresPermissions("system:business:remove")
    @Log(title = "企业", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public Result remove(@PathVariable Long[] ids)
    {
        return toAjax(businessService.deleteBusinessByIds(ids));
    }
}
