package com.zhiLian.business.service.impl;


import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import com.zhiLian.business.domain.Business;

import com.zhiLian.business.mapper.BusinessMapper;
//import com.zhiLian.business.remote.factory.RemoteUserLoginFactory;
import com.zhiLian.business.service.IBusinessService;
import com.zhiLian.common.business.domain.Entinfo;
import com.zhiLian.common.core.domain.Result;
import com.zhiLian.common.core.utils.DateUtils;
import com.zhiLian.common.redis.service.RedisService;
import com.zhiLian.common.security.utils.SecurityUtils;
import com.zhiLian.common.system.domain.LoginUser;
import com.zhiLian.common.system.domain.SysUser;
import com.zhiLian.common.system.remote.RemoteUserService;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.HashMap;
import java.util.List;
import java.util.UUID;

/**
 * 企业Service业务层处理
 *
 * @author muyu
 * @date 2024-05-26
 */
@Service
public class BusinessServiceImpl extends ServiceImpl<BusinessMapper, Business>
        implements IBusinessService
{
    @Autowired
    private BusinessMapper businessMapper;

    @Autowired
    private RemoteUserService remoteUserService;

    @Autowired
    private RedisService redisService;

    /**
     * rabbitMq 发送消息
     */
    @Autowired
    private RabbitTemplate rabbitTemplate;

    /**
     * 查询企业
     *
     * @param id 企业主键
     * @return 企业
     */
    @Override
    public Business selectBusinessById(Long id)
    {
        return businessMapper.selectBusinessById(id);
    }


    /**
     * 查询企业列表
     *
     * @param business 企业
     * @return 企业
     */
    @Override
    public List<Business> selectBusinessList(Business business)
    {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        SysUser user = remoteUserService.selectByUserId(loginUser.getUserid());
        if (user.getUserType().equals("00")) {
            return businessMapper.selectBusinessList(business);
        }
        business.setId(Long.valueOf(user.getUserType()));
        return businessMapper.selectBusinessList(business);
    }

    /**
     * 新增企业
     *
     * @param business 企业
     * @return 结果
     */
//    @Autowired
//    private RemoteUserLoginFactory remoteUserLoginFactory;
    @Override
    public int insertBusiness(Business business)
    {
        business.setCreateTime(DateUtils.getNowDate());
        int i = businessMapper.insertBusiness(business);
        System.out.println(business);
        SysUser sysUser = SysUser.builder()
                .userName(business.getName())
                .password("admin123")
                .nickName(business.getName())
                .email(business.getBusinessEmail())
                .phonenumber(business.getBusinessPhone())
                .loginIp("111.229.102.61")
                .roleId(101L)
                .userType(String.valueOf(business.getId()))
                .build();
        Result add = remoteUserService.add(sysUser);
        return i;
    }


    /**
     * 修改企业
     *
     * @param business 企业
     * @return 结果
     */
    @Override
    public int updateBusiness(Business business)
    {
        business.setUpdateTime(DateUtils.getNowDate());
        httpConnectMysql(business);
        return businessMapper.updateBusiness(business);
    }
    @Autowired
    private EntinfoServiceImpl entinfoService;

    /**
     * 判断redis是否存在端口号
     * @param business
     */
    public void httpConnectMysql(Business business){
        redisService.deleteObject("entinfo");
        List<Entinfo> list = entinfoService.list();
        if(list.size()!=0){
            redisService.setCacheList("entinfo",list);
        }
        if (business.getBusinessStates().equals("2")) {
            SysUser sysUser = remoteUserService.userById(business.getId());
            sysUser.setRoleId(2L);
            remoteUserService.updateById(sysUser);
            Entinfo build = Entinfo.builder()
                    .entCode("test_" + business.getId())
                    .ip("192.168.120.128")
                    .port(3306 + Integer.parseInt(String.valueOf(business.getId()))).build();
            Entinfo entinfo =entinfoService.selectEntinfo(build);
            if (entinfo==null){
                extracted(business);
            }
        }
    }

    /**
     * http URL 进行创建Mysql  Docker
     * @param business
     */
    private  void extracted(Business business) {
        String postUrl="http://192.168.120.128/webhook/%E6%96%B0%E5%BB%BAmysql%E6%9C%8D%E5%8A%A1";
        HashMap<String, String> hashMap = new HashMap<>();
        hashMap.put("businessId",business.getId()+ business.getName());
        hashMap.put("mysqlPort",String.valueOf(3306+ business.getId()));
        String json = JSON.toJSONString(hashMap);
        // 3.创建连接与设置连接参数
        URL urlObj = null;
        try {
            urlObj = new URL(postUrl);
            HttpURLConnection httpConn = (HttpURLConnection) urlObj.openConnection();
            httpConn.setRequestMethod("POST");
            httpConn.setRequestProperty("Charset", "UTF-8");
            // POST请求且JSON数据,必须设置
            httpConn.setRequestProperty("Content-Type", "application/json");
            // 打开输出流,默认是false
            httpConn.setDoOutput(true);
//            // 打开输入流,默认是true,可省略
//            httpConn.setDoInput(true);
            // 4.从HttpURLConnection获取输出流和写数据
            OutputStream oStream = httpConn.getOutputStream();
            oStream.write(json.getBytes());
            oStream.flush();
            // 5.发起http调用(getInputStream触发http请求)
            if (httpConn.getResponseCode() != 200) {
                throw new Exception("调用服务端异常.");
            }
            // 6.从HttpURLConnection获取输入流和读数据
            BufferedReader br = new BufferedReader(
                    new InputStreamReader(httpConn.getInputStream()));
            String resultData = br.readLine();
            System.out.println("从服务端返回结果: " + resultData);
            Entinfo build = Entinfo.builder()
                    .entCode("test_" + business.getId())
                    .ip("192.168.120.128")
                    .port(3306 + Integer.parseInt(String.valueOf(business.getId()))).build();
            entinfoService.insertEntinfo(build);
            rabbitTemplate.convertAndSend("zhiLian-vehicle-exchange",JSON.toJSONString(build),message ->{
                message.getMessageProperties().setMessageId(UUID.randomUUID().toString());
                //设置消息延迟时间为5秒
                message.getMessageProperties().setDelay(5000);
                return message;
            } );
            // 7.关闭HttpURLConnection连接
            httpConn.disconnect();
        } catch (Exception e) {
            throw new RuntimeException(e);
        }


    }

    /**
     * 批量删除企业
     *
     * @param ids 需要删除的企业主键
     * @return 结果
     */
    @Override
    public int deleteBusinessByIds(Long[] ids)
    {
        return businessMapper.deleteBusinessByIds(ids);
    }

    /**
     * 删除企业信息
     *
     * @param id 企业主键
     * @return 结果
     */
    @Override
    public int deleteBusinessById(Long id)
    {
        return businessMapper.deleteBusinessById(id);
    }
}
