package com.zhiLian.iotdb.controller;


/**
 * BingRui.Hou
 *
 * @Description 描述
 * @ClassName IotDbController
 * @Date 2024/05/21 16:55
 */

import com.bwie.common.result.Result;
import com.zhiLian.iotdb.config.IotDBSessionConfig;
import com.zhiLian.iotdb.domain.IotDbParam;
import com.zhiLian.iotdb.service.IotDbServer;
import lombok.extern.log4j.Log4j2;
import org.apache.iotdb.rpc.IoTDBConnectionException;
import org.apache.iotdb.rpc.StatementExecutionException;
import org.apache.kafka.clients.producer.KafkaProducer;
import org.apache.kafka.clients.producer.ProducerConfig;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.apache.kafka.common.serialization.StringSerializer;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.rmi.ServerException;
import java.util.Properties;

/**
 * description: iotdb 控制层
 * date: 2022/8/15 21:50
 * author: zhouhong
 */
@Log4j2
@RestController
public class IotDbController {

    @Resource
    private IotDbServer iotDbServer;
    @Resource
    private IotDBSessionConfig iotDBSessionConfig;

    /**
     * 插入数据
     * @param iotDbParam
     */
    @PostMapping("/api/device/insert")
    public Result insert(@RequestBody IotDbParam iotDbParam) throws StatementExecutionException, ServerException, IoTDBConnectionException {
        iotDbServer.insertData(iotDbParam);
        return Result.success();
    }

    @GetMapping("getMsg")
    public void  getMsg(@RequestParam("mgs") String msg){
        //TODO 发送消息 Kafka
        //TODO 用来配置 kafka消息生产者对象的配置信息
        Properties properties = new Properties();
        //TODO 配置 host
        properties.setProperty(ProducerConfig.BOOTSTRAP_SERVERS_CONFIG, "122.51.111.225:9092");
        //TODO 配置 键值的序列化方式
        properties.setProperty(ProducerConfig.KEY_SERIALIZER_CLASS_CONFIG, StringSerializer.class.getName());
        properties.setProperty(ProducerConfig.VALUE_SERIALIZER_CLASS_CONFIG, StringSerializer.class.getName());
        //TODO 配置消息的确认
        properties.setProperty(ProducerConfig.ACKS_CONFIG, "all");
        //TODO 设置重试次数
        properties.setProperty(ProducerConfig.RETRIES_CONFIG, "3");
        //TODO 消息压缩
        properties.setProperty(ProducerConfig.COMPRESSION_TYPE_CONFIG, "gzip");
        //TODO 创建消息生产者对象
        KafkaProducer<String, String> kafkaProducer = new KafkaProducer<String, String>(properties);

        //TODO 发送消息
        //TODO 创建 消息记录
        ProducerRecord<String, String> producerRecord = new ProducerRecord<String, String>("bwie-topic", "key", msg);
        // TODO       kafkaProducer.send(producerRecord);

        //TODO 异步发送
        kafkaProducer.send(producerRecord, (metadata, exception) -> {
            if (exception == null) {
                //TODO 获取消息发送的分区
                int partition = metadata.partition();
                // TODO 消息的偏移量
                long offset = metadata.offset();
                //TODO 主题
                String topic = metadata.topic();
                System.out.println("发送成功，消息的分区：" + partition + "，消息的偏移量：" + offset + "，主题：" + topic);
            } else {
                System.out.println("发送失败，异常信息：" + exception);
            }
        });

        //TODO 关闭 kafkaProducer
        kafkaProducer.close();
    }
    /**
     * 查询数据
     * @param iotDbParam
     */
    @PostMapping("/api/device/queryData")
    public Result queryDataFromIotDb(@RequestBody IotDbParam iotDbParam) throws Exception {
        return Result.success(iotDbServer.queryDataFromIotDb(iotDbParam));
    }
    /**
     * 删除分组
     * @return
     */
    @PostMapping("/api/device/deleteGroup")
    public Result deleteGroup() throws StatementExecutionException, IoTDBConnectionException {
        iotDBSessionConfig.deleteStorageGroup("root.a1eaKSRpRty");
        iotDBSessionConfig.deleteStorageGroup("root.smartretirement");
        return Result.success();
    }

}
