package com.zhiLian.vehicle.datasource;


import com.alibaba.druid.pool.DruidDataSource;
import com.alibaba.fastjson2.JSON;
import com.rabbitmq.client.Channel;
import com.zhiLian.common.business.domain.Entinfo;
import com.zhiLian.common.business.remote.RemoteBusinessService;
import com.zhiLian.common.core.utils.SpringUtils;
import com.zhiLian.common.redis.service.RedisService;
import com.zhiLian.vehicle.datasource.config.factory.DruidDataSourceFactory;
import com.zhiLian.vehicle.datasource.config.role.DynamicDataSource;
import com.zhiLian.vehicle.datasource.domain.DataSourceInfo;
import lombok.extern.log4j.Log4j2;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.Queue;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.AutoConfigureBefore;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Lazy;
import org.springframework.context.annotation.Primary;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 *  BingRui.Hou
 *  数据源配置
 */
@Log4j2
@Component
@AutoConfigureBefore(RedisService.class)
public class ManyDataSource {


    @Autowired
    private RedisTemplate<String,String> redisTemplate;

    @Autowired
    private RemoteBusinessService remoteBusinessService;



    //调用注解  添加队列名称
    @RabbitListener(queuesToDeclare = {@Queue(name = "zhiLian-vehicle-exchange")})
    public void smsConfig(String msg, Message message, Channel channel){
        //获取消息的ID
        String messageId = message.getMessageProperties().getMessageId();
        try {
            //添加消息id到redis set集合中  添加成功返回1 表示未消费    添加失败返回0 表示已消费
            Long count = redisTemplate.opsForSet().add("messageId", messageId);
            //添加成功  正常消费信息
            if (count == 1) {
                log.info("开始消费");
                druidData(msg);
                //确认消费
                channel.basicAck(message.getMessageProperties().getDeliveryTag(),false);
                log.info("消费成功");
            }
        } catch (Exception e) {
            log.info("消费失败，尝试重连");
            try {
                //回退消息,尝试重连
                druidData(msg);
                channel.basicReject(message.getMessageProperties().getDeliveryTag(),false);
                log.info("消费失败");
            } catch (IOException ex) {
                //回退失败
                log.info("消费异常");
            }
        }
    }

    private void druidData(String msg) {
        Entinfo entinfo1 = JSON.parseObject(msg, Entinfo.class);
        log.info("消息为：{}",entinfo1);
        DruidDataSourceFactory druidDataSourceFactory = SpringUtils.getBean(DruidDataSourceFactory.class);
        DynamicDataSource dynamicDataSource = SpringUtils.getBean(DynamicDataSource.class);
        DataSourceInfo dataSourceInfo = DataSourceInfo.hostAndPortBuild(entinfo1.getEntCode(), entinfo1.getIp(), entinfo1.getPort());
        DruidDataSource druidDataSource = druidDataSourceFactory.create(dataSourceInfo);
        dynamicDataSource.put(dataSourceInfo.getKey(), druidDataSource);
    }

    @Lazy
    private List<Entinfo> dataSourceInfoList(){
        List<Entinfo> databaseNameList = new ArrayList<>(){{
            add(Entinfo.builder()
                    .entCode("test_00")
                    .ip("192.168.120.128")
                    .port(3306)
                    .build());
        }};
        List<Entinfo> list = remoteBusinessService.listAll();
        databaseNameList.addAll(list);
        return databaseNameList;
    }



    @Bean
    @Primary
    public DynamicDataSource dynamicDataSource(DruidDataSourceFactory druidDataSourceFactory) {
        //查企业
        Map<Object, Object> dataSourceMap = new HashMap<>();
        dataSourceInfoList()
                .stream()
                .map(entInfo-> DataSourceInfo.hostAndPortBuild(entInfo.getEntCode(),entInfo.getIp(),entInfo.getPort()))
                .forEach(dataSourceInfo -> {
            dataSourceMap.put(dataSourceInfo.getKey(), druidDataSourceFactory.create(dataSourceInfo));
        });
        //设置动态数据源
        DynamicDataSource dynamicDataSource = new DynamicDataSource();
//        dynamicDataSource.setDefaultTargetDataSource(masterDataSource());
        dynamicDataSource.setTargetDataSources(dataSourceMap);
        //将数据源信息备份在defineTargetDataSources中
        dynamicDataSource.setDefineTargetDataSources(dataSourceMap);
        return dynamicDataSource;
    }
}
