package com.zhiLian.vehicle.datasource.config.role;


import com.alibaba.druid.pool.DruidDataSource;
import com.zhiLian.vehicle.datasource.config.holder.DynamicDataSourceHolder;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import org.springframework.jdbc.datasource.lookup.AbstractRoutingDataSource;

import java.util.Map;

/**
 * 动态数据源
 * 调用AddDefineDataSource组件的addDefineDynamicDataSource（）方法，获取原来targetdatasources的map，并将新的数据源信息添加到map中，并替换targetdatasources中的map
 * 切换数据源时可以使用@DataSource(value = "数据源名称")，或者DynamicDataSourceContextHolder.setContextKey("数据源名称")
 * @author BingRui.Hou
 */
@Data
@AllArgsConstructor
@NoArgsConstructor
@EqualsAndHashCode(callSuper = true)
public class DynamicDataSource extends AbstractRoutingDataSource {
    /**
     * 备份所有数据源信息，   指针！！
     */
    private Map<Object, Object> defineTargetDataSources;


    /**
     * 决定当前线程使用哪个数据源
     */
    @Override
    protected Object determineCurrentLookupKey() {
        return DynamicDataSourceHolder.getDynamicDataSourceKey();
    }


    /**
     * 添加数据库
     * @param key 键
     * @param value 数据源
     */
    public void put(String key, DruidDataSource value) {
        defineTargetDataSources.put(key, value);
        this.afterPropertiesSet();
    }
}
